<?php

/**
 * @file
 * Contains \Drupal\chessboard_filter\ChessboardBulletinBoardCodeTagContentFilter.
 */

namespace Drupal\chessboard_filter;

use Drupal\chessboard\ChessboardFilterInterface as FilterInterface;
use Drupal\chessboard_images\Diagram;
use Drupal\chessboard_images\PiecePlacement;

/**
 * Provides chessboard BBCode tag rendering.
 *
 * @internal
 */
class ChessboardBulletinBoardCodeTagContentFilter implements FilterInterface{

  /**
  * Filter options.
   *
   * @var array
   */
  protected $options;

  /**
   * Constructs a Drupal\chessboard_filter\ChessboardBulletinBoardCodeTagContentFilter object.
   *
   * @param array $options
   *   A filter options array.
   */
  public function __construct(array $options) {
    $this->options = $options;
  }

  /**
   * Returns the result of filtering $value.
   *
   * @param string $value
   *   The piece placement field.
   * @return array
   *   A render <code>array</code> containing the replacement text.
   */
  public function filter($value) {
    $item = $this->parse($value);
    return array(
      '#lazy_builder' => array(
        function($board_value, $langcode, $file_max, $border_t, $border_b, $border_l, $border_r, $square_color_first) {
          $diagram = new Diagram();
          $element = $diagram->setPiecePlacement(new PiecePlacement($board_value, $file_max))
            ->setBorder(array('T' => $border_t, 'B' => $border_b, 'L' => $border_l, 'R' => $border_r))
            ->setSquareColorFirst($square_color_first)
            ->toRenderable();

          // @todo Remove once https://www.drupal.org/node/2609250 is in.
          $element += \Drupal::service('element_info')->getInfo($element['#type']);

          return $element;
        },
        // @todo Pass along langcode.
        array($item['board_value'], '', $item['file_max'], $item['border']['T'], $item['border']['B'], $item['border']['L'], $item['border']['R'], $item['square_color_first']),
      ),
    );
  }

  private function parse($value) {
    preg_match('@^\s*(?:\((.*?)\))?(.*)$@s', $value, $matches);
    $item = array();

    // Parameter Parsing
    //--------------------------------
    $params =& $matches[1];

    // Number of files: any integer.
    if (preg_match('@[0-9]+@', $params, $m)) {
      $item['file_max'] = $m[0];
    }

    // Color of the upper left square: [l]ight or [d]ark.
    if (strpos($params, 'd') !== FALSE) {
      $item['square_color_first'] = 1;
    }
    elseif (strpos($params, 'l') !== FALSE) {
      $item['square_color_first'] = 0;
    }

    // Borders.
    $item['border'] = array('T' => (strpos($params, 'T') !== FALSE),
                    'B' => (strpos($params, 'B') !== FALSE),
                    'L' => (strpos($params, 'L') !== FALSE),
                    'R' => (strpos($params, 'R') !== FALSE));

    $item += $this->options;

    // Render the board in XHTML syntax
    //----------------------------------
    $board_value =& $matches[2];
    $board_value = strip_tags($board_value);
    // Ignore unknown characters.
    $board_value = preg_replace('@[^-0-9KQBNRPACkqbnrpacx]@', '', $board_value);
    // Multiple empty squares.
    $replace_pairs = array();
    for ($i=10; $i >= 0; $i--) {
      $replace_pairs[$i] = str_repeat('-', $i);
    }
    $board_value = strtr($board_value, $replace_pairs);
    $item['board_value'] = $board_value;

    return $item;
  }

}
