<?php

/**
 * @file
 * Contains \Drupal\chessboard_images\Element\ChessboardImagesDiagram.
 */

namespace Drupal\chessboard_images\Element;

use Drupal\Core\Render\Element\RenderElement;

/**
 * @RenderElement("chessboard_images_diagram")
 */
class ChessboardImagesDiagram extends RenderElement {

  public function getInfo() {
    $image_directory_path = drupal_get_path('module', 'chessboard_images') . '/default';
    $dimensions = array(
      'square' => array(
        'width' => 40,
        'height' => 40,
      ),
      'border-h' => array(
        'width' => 40,
        'height' => 4,
      ),
      'border-v' => array(
        'width' => 4,
        'height' => 40,
      ),
      'border-c' => array(
        'width' => 4,
        'height' => 4,
      ),
    );
    $class = get_class($this);
    $type = array(
      '#image_directory_path' => $image_directory_path,
      '#pre_render' => array(
        array($class, 'preRenderChessboardDiagram'),
      ),
      '#theme' => 'chessboard_images_diagram',
    );
    // Construct the translation table
    $size_attributes = $dimensions;

    // Empty and marked squares
    $table_tiles = array(
      '-0' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->pieceFilename('-', 0), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => '-', '#title' => ''),
      '-1' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->pieceFilename('-', 1), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => '-', '#title' => ''),
      'x0' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->pieceFilename('x', 0), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => 'x', '#title' => ''),
      'x1' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->pieceFilename('x', 1), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => 'x', '#title' => ''),
    );

    // Pieces
    $pieces = 'kqbnrpac';
    for ($i=0; $i<8; $i++) {
      $piece = $pieces[$i];
      $table_tiles += array(
        strtoupper($piece) . '0' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->pieceFilename(strtoupper($piece), 0), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => strtoupper($piece), '#title' => ''),
        strtoupper($piece) . '1' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->pieceFilename(strtoupper($piece), 1), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => strtoupper($piece), '#title' => ''),
        $piece . '0' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->pieceFilename($piece, 0), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => $piece, '#title' => ''),
        $piece . '1' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->pieceFilename($piece, 1), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => $piece, '#title' => ''),
      );
    }

    // Borders
    $table_tiles += array(
      'T' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->borderFilename('T'), '#width' => $size_attributes['border-h']['width'], '#height' => $size_attributes['border-h']['height'], '#title' => ''),
      'B' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->borderFilename('B'), '#width' => $size_attributes['border-h']['width'], '#height' => $size_attributes['border-h']['height'], '#title' => ''),
      'L' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->borderFilename('L'), '#width' => $size_attributes['border-v']['width'], '#height' => $size_attributes['border-v']['height'], '#title' => ''),
      'R' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->borderFilename('R'), '#width' => $size_attributes['border-v']['width'], '#height' => $size_attributes['border-v']['height'], '#title' => ''),
      'TL' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->borderFilename('TL'), '#width' => $size_attributes['border-c']['width'], '#height' => $size_attributes['border-c']['height'], '#title' => ''),
      'TR' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->borderFilename('TR'), '#width' => $size_attributes['border-c']['width'], '#height' => $size_attributes['border-c']['height'], '#title' => ''),
      'BL' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->borderFilename('BL'), '#width' => $size_attributes['border-c']['width'], '#height' => $size_attributes['border-c']['height'], '#title' => ''),
      'BR' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . $this->borderFilename('BR'), '#width' => $size_attributes['border-c']['width'], '#height' => $size_attributes['border-c']['height'], '#title' => ''),
    );
    $type['#table_tiles'] = $table_tiles;
    return $type;
  }

  public static function preRenderChessboardDiagram($elements) {
    $diagram = $elements['#diagram'];
    $elements['#border'] = $diagram->getBorder();
    $elements['#square_color_first'] = $diagram->getSquareColorFirst();
    $piece_placement = $diagram->getPiecePlacement();
    $elements['#file_max'] = $piece_placement->getFileMax();
    $elements['#board'] = array();
    $size = $piece_placement->getSize();
    for ($i = 0; $i < $size; $i++) {
      $rank_d = (int) floor($i / $elements['#file_max']);
      $file = $i - $rank_d * $elements['#file_max'];
      if ($file == 0) {
        $elements['#board'][$rank_d] = array();
      }
      $elements['#board'][$rank_d][$file] = $piece_placement->getPieceCode($rank_d, $file);
    }
    return $elements;
  }

  protected function borderFilename($border) {
    // $border: T, B, L, R, TL, TR, BL, BR
    switch ($border) {
      case 'T':
      case 'B':
        return 'h.png';
      case 'L':
      case 'R':
        return 'v.png';
      default:
        return 'c.png';
    }
  }

  protected function pieceFilename($piece, $square_color) {
    // $piece        : KQBNRPkqbnrp-
    // $square_color : 0 1
    switch ($piece) {
      case 'x':
        $name = 'xx';
        break;
      case '-':
        $name = '';
        break;
      default:
        $name = strtolower($piece) . (ctype_lower($piece) ? 'd' : 'l');
        break;
    }

    return $name . ($square_color ? 'd' : 'l') . '.png';
  }

}
