<?php

/**
 * @file
 * Contains \Drupal\chessboard_images_formatter\Plugin\Field\FieldFormatter\ChessboardImagesFormatter.
 */

namespace Drupal\chessboard_images_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\chessboard_images_formatter\PiecePlacementBuilder;
use Drupal\chessboard_images\Diagram;

/**
 * Plugin implementation of the 'chessboard_images_formatter_diagram' formatter.
 *
 * @FieldFormatter(
 *   id = "chessboard_images_formatter_diagram",
 *   label = @Translation("Chessboard images"),
 *   field_types = {
 *     "chessboard"
 *   }
 * )
 */
class ChessboardImagesFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = array();

    foreach ($items as $delta => $item) {
      $elements[$delta] = array(
        '#item' => $item,
        '#langcode' => $langcode,
        '#pre_render' => array(array(static::class, 'preRenderPiecePlacement')),
      );
    }

    return $elements;
  }

  /**
   * Pre-render callback.
   *
   * @param array $element
   *   A structured array with the following key-value pairs:
   *   - #item: \Drupal\chessboard\Plugin\Field\FieldType\ChessboardItem
   *
   * @return array
   */
  public static function preRenderPiecePlacement(array $element) {
    $element['piece_placement'] = array(
      '#lazy_builder' => array(
        array(static::class, 'buildPiecePlacement'),
        array($element['#item']->get('piece_placement')->getString(), $element['#langcode']),
      ),
    );
    return $element;
  }

  /**
   * Lazy builder callback.
   *
   * @param string $value
   * @param string $langcode
   *
   * @return array
   */
  public static function buildPiecePlacement($value, $langcode) {
    $builder = new PiecePlacementBuilder();
    $builder->setFileMax(8);
    $diagram = new Diagram();
    $element = $diagram->setPiecePlacement($builder->setValue($value)->build())->toRenderable();
    // @todo Remove once https://www.drupal.org/node/2609250 is in.
    $element += \Drupal::service('element_info')->getInfo($element['#type']);
    return $element;
  }

}
