<?php

/**
 * @file
 * Contains \Drupal\chessboard\Plugin\Field\FieldFormatter\ChessboardSimpleFormatter.
 */

namespace Drupal\chessboard\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\SafeMarkup;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'chessboard simple' formatter.
 *
 * @FieldFormatter(
 *   id = "chessboard_simple",
 *   label = @Translation("Chessboard simple"),
 *   field_types = {
 *     "chessboard"
 *   }
 * )
 */
class ChessboardSimpleFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = array();

    foreach ($items as $delta => $item) {
      $elements[$delta] = array(
        '#prefix' => '<pre>',
        '#item' => $item,
        '#langcode' => $langcode,
        '#pre_render' => array(array(static::class, 'preRenderPiecePlacement')),
        '#suffix' => '</pre>',
      );
    }

    return $elements;
  }

  /**
   * Pre-render callback.
   *
   * @param array $element
   *   A structured array with the following key-value pairs:
   *   - #item: \Drupal\chessboard\Plugin\Field\FieldType\ChessboardItem
   *
   * @return array
   */
  public static function preRenderPiecePlacement(array $element) {
    $element['piece_placement'] = array(
      '#lazy_builder' => array(
        function($piece_placement, $langcode) { return array('#plain_text' => chunk_split($piece_placement, 8, "\n")); },
        array($element['#item']->get('piece_placement')->getString(), $element['#langcode']),
      ),
    );
    return $element;
  }

}
